package systemdCheck::SmeSystemdCheck;

=head1 NAME

systemdCheck::SmeSystemdCheck; - Check for Systemd services

=head1 VERSION

version 1.00

=head1 SYNOPSIS

Check for Systemd services

Shamelessly copied from

<https://github.com/orkunkaraduman/Zabbix-Check>
<https://metacpan.org/release/Zabbix-Check>

=head1 AUTHOR

Orkun Karaduman (ORKUN) <orkun@cpan.org>
John Crisp <jcrisp@safeandsounit.co.uk>

=cut

use strict;
use warnings;
use v5.16.3;
use JSON;
use Lazy::Utils;

BEGIN {
    require Exporter;
    our $VERSION = '1.00';
    our @ISA     = qw(Exporter);
    our @EXPORT  = qw(installed system_status service_discovery service_discovery_json service_status version);
    our @EXPORT_OK = qw();
}

our ($systemctl) = whereis('systemctl');

# Check is systemd is installed
sub installed {
    my $result = $systemctl ? 1 : 0;
    return $result;
}

# Returns systemd service status
sub system_status {
    my $result = "";
    my $line = `$systemctl is-system-running 2>/dev/null` if $systemctl;
    if ($line) {
        chomp $line;
        $result = $line;
    }
    return $result;
}

# Returns an array grepped from the service name
sub service_discovery {
    my $service = shift @_;
    return "" unless $service;
    my ($name_rgx) = map( zbx_decode($_), $service );
    my @items;
    my $units = get_units('service');
    @items = map( $units->{$_},
        grep(
            { not defined($name_rgx) or $units->{$_}->{name} =~ /$name_rgx/ }
            keys %$units ) )
        if $units;
    return @items;
}

# Returns an array grepped from the service name
sub service_discovery_json {
    my $service = shift @_;
    return "" unless $service;
    my ($name_rgx) = map( zbx_decode($_), $service );
    my @items;
    my $units = get_units('service');
    @items = map( $units->{$_},
        grep(
            { not defined($name_rgx) or $units->{$_}->{name} =~ /$name_rgx/ }
            keys %$units ) )
        if $units;
    return json_discovery(@items);
}

# Returns status of the service name
sub service_status {
    my $service = shift @_;
    return "" unless $service;
    my ($name) = map( zbx_decode($_), $service );
    return "" unless $name;
    my $name_s = shellmeta($name);
    my $result = "";
    my $line   = `$systemctl is-active \"$name_s.service\" 2>/dev/null`
        if $systemctl;
    if ($line) {
        chomp $line;
        $result = $line;
    }
    return $result;
}

# Sub routines
our @zbx_specials
    = split( " ", q%\ ' " ` * ? [ ] { } ~ $ ! & ; ( ) < > | \# @% );

sub zbx_encode {
    my $result = "";
    my ($str) = @_;
    return $result unless defined($str);
    for ( my $i = 0; $i < length $str; $i++ ) {
        my $chr = substr $str, $i, 1;
        if ( not $chr =~ /[ -~]/g
            or grep( $_ eq $chr, ( @zbx_specials, '%', ',' ) ) )
        {
            $result .= uc sprintf( "%%%x", ord($chr) );
        }
        else {
            $result .= $chr;
        }
    }
    return $result;
}

sub zbx_decode {
    my $result = "";
    my ($str) = @_;
    return $result unless defined($str);
    my ( $i, $len ) = ( 0, length $str );
    while ( $i < $len ) {
        my $chr = substr $str, $i, 1;
        if ( $chr eq '%' ) {
            return $result if $len - $i - 1 < 2;
            $result .= chr( hex( substr( $str, $i + 1, 2 ) ) );
            $i += 2;
        }
        else {
            $result .= $chr;
        }
        $i++;
    }
    return $result;
}

sub json_discovery {
    my @items     = @_;
    my $discovery = {
        data => [
            map( {
                    my $item    = $_;
                    my %newitem = map( {
                            my $key    = $_;
                            my $val    = $item->{$key};
                            my $newkey = zbx_encode($key);
                            $newkey = uc("{#$newkey}");
                            my $newval = zbx_encode($val);
                            $newkey => $newval;
                    } keys(%$item) );
                    \%newitem;
            } @items ),
        ],
    };
    my $result = to_json( $discovery, { pretty => 1 } );
    return $result;
}

sub version {
    my $result = "";
    $result = $systemdCheck::SmeSystemdCheck::VERSION;
    return $result;
}

sub get_unit_files {
    return unless $systemctl;
    my ($type) = @_;
    my $result = {};
    for (`$systemctl --no-legend list-unit-files 2>/dev/null`) {
        $_ = trim($_);
        last unless $_;
        my ( $unit, $state ) = /^(\S+)\s+(\S+)/;
        my $info = {
            unit  => $unit,
            state => $state,
        };
        ( $info->{name}, $info->{type} ) = $unit =~ /^([^\.]*)\.(.*)/;
        $result->{$unit} = $info if not $type or $type eq $info->{type};
    }
    return $result;
}

sub get_units {
    return unless $systemctl;
    my ($type) = @_;
    my $result = {};
    my $first  = 1;
    for (`$systemctl --no-legend -a list-units 2>/dev/null`) {
        $_ = trim($_);
        last unless $_;
        my ( $unit, $load, $active, $sub, $desc )
            = /^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/;
        my $info = {
            unit   => $unit,
            load   => $load,
            active => $active,
            sub    => $sub,
            desc   => $desc,
        };
        ( $info->{name}, $info->{type} ) = $unit =~ /^([^\.]*)\.(.*)/;
        $result->{$unit} = $info if not $type or $type eq $info->{type};
    }
    return $result;
}

1;
__END__
=head1 REPOSITORY

B<GitHub> L<https://github.com/orkunkaraduman/Zabbix-Check>

B<CPAN> L<https://metacpan.org/release/Zabbix-Check>

=head1 AUTHOR

Orkun Karaduman (ORKUN) <orkun@cpan.org>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2017  Orkun Karaduman <orkunkaraduman@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

